/*
 * Enhydra Java Application Server
 * The Initial Developer of the Original Code is Lutris Technologies Inc.
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc.
 * All Rights Reserved.
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License.
 *
 * $Id: FormServlet.java,v 1.1 2000/07/22 00:44:49 pmahar Exp $
 */

package kelp.webapp.presentation;

import org.w3c.dom.html.HTMLInputElement;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import java.io.IOException;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.Properties;

/**
 * <p>
 * This presentation object dynamically creates an HTML page
 * showing the greeting from the application configuration file.
 * </p>
 */
public class FormServlet extends HttpServlet {
  private String     dataFilename = null;
  private Properties data         = null;
  private FormHTML   page         = null;

  public void doPost(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException {
    doGet(request, response);
  }

  public void doGet(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
    initData(request);
    saveParameters(request);
    page = createPage();

    response.getWriter().write(page.toDocument());
    response.getWriter().flush();
    response.flushBuffer();
  }

  private void saveParameters(HttpServletRequest request) {
    String paramFirst = null;
    String paramLast = null;
    try {
      paramFirst = request.getParameter("first");
      paramLast =  request.getParameter("last");
    } catch (Exception e) {
      paramFirst = null;
      paramLast  = null;
    }
    if (paramFirst != null && paramLast != null) {
      data.put("first", paramFirst);
      data.put("last", paramLast);
      writePropertyFile();
    }
  }

  public FormHTML createPage()  {
    page = new FormHTML();
    fixupForm();
    return page;
  }

  private void fixupForm() {
    String first = (String) data.get("first");
    String last  = (String) data.get("last");
    HTMLInputElement inputFirst = page.getElementFirst();
    HTMLInputElement inputLast  = page.getElementLast();
    inputFirst.setValue(first);
    inputLast.setValue(last);
  }


  private void writePropertyFile() {
    FileOutputStream out;
    try {
      out = new FileOutputStream(dataFilename, false);
      data.save(out, "Data for Form page");
      out.close();
    } catch (java.io.IOException e) {
      e.printStackTrace();
    }
  }

  private void readPropertyFile() {
    FileInputStream is = null;
    try {
      is = new FileInputStream(dataFilename);
      data = new Properties();
      data.load(is);
      is.close();
    } catch (Exception e) {
      // create deault
      e.printStackTrace();
      data = new Properties();
      data.put("first","Luke");
      data.put("last","Skywalker");
    }
  }

  private void initData(HttpServletRequest request) {
    StringBuffer fileNameBuf = new StringBuffer();
    String testPath = getInitParameter("testDataPath");
    if (testPath == null) {
      System.out.println("init-parm not found: testDataPath");
    } else {
      fileNameBuf.append(testPath);
      fileNameBuf.append(File.separator);
      fileNameBuf.append("form.properties");
      dataFilename = fileNameBuf.toString();
    }
    data = (Properties) request.getSession().getAttribute("data");
    if (data == null) {
      readPropertyFile();
    }
  }
}


