/*
 * Enhydra Java Application Server
 * The Initial Developer of the Original Code is Lutris Technologies Inc.
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc.
 * All Rights Reserved.
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License.
 *
 * $Id: ValidationUtil.java,v 1.1 2000/05/30 07:24:26 pmahar Exp $
 */
package org.enhydra.kelp.common;
import java.io.File;
import java.util.StringTokenizer;

public class ValidationUtil {

    /**
     * Hidden constructor
     */
    private ValidationUtil() {}

    /**
     * Test to see if the given package is a valid java package name.
     *
     * @param pack
     * The package name to test
     *
     * @return
     * True if the specified package name is valid.
     */
    public static boolean isJavaPackage(String pack) {
        boolean         valid = true;
        StringTokenizer tokenizer = null;

        if (pack.trim().startsWith(".") || pack.trim().endsWith(".")) {
            valid = false;
        } else {
            tokenizer = new StringTokenizer(pack, ".");
            while (tokenizer.hasMoreTokens()) {
                if (!ValidationUtil.isJavaIdentifier(tokenizer.nextToken())) {
                    valid = false;
                    break;
                }
            }
        }
        return valid;
    }

    /**
     * Test to see if the given indentifier is a valid java identifier.
     *
     * @param ident
     * The identifier to test.
     *
     * @return
     * True if the specified identifier is valid.
     */
    public static boolean isJavaIdentifier(String ident) {
        boolean valid = true;

        for (int i = 0; i < ident.length(); i++) {
            char ch = ident.charAt(i);

            if (i == 0) {
                if (!Character.isJavaIdentifierStart(ch)) {
                    valid = false;
                    break;
                }
            } else if (!Character.isJavaIdentifierPart(ch)) {
                valid = false;
                break;
            }
        }
        return valid;
    }

    /**
     * Test to see if a path is a directory.
     *
     * @param path
     * The path to test.
     *
     * @return
     * True if the specified path is a directory.
     */
    public static boolean isDirectory(String path) {
        File file = new File(path);
        return ValidationUtil.isDirectory(file);
    }

    public static boolean isDirectory(File file) {
        boolean valid = false;

        if (file != null) {
            valid = file.isDirectory();
        }
        return valid;
    }

    /**
     * Test to see if the parent of a given path is a valid directory.
     *
     * @param path
     * The path to test.
     *
     * @return
     * True if the parent of the specified path is a directory.
     */
    public static boolean isParentDirectory(String path) {
        boolean valid = false;
        File    file = new File(path);

        if (file != null && file.getParentFile() != null) {
            valid = file.getParentFile().isDirectory();
        }
        return valid;
    }

}
