/*
 * Enhydra Java Application Server
 * The Initial Developer of the Original Code is Lutris Technologies Inc.
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc.
 * All Rights Reserved.
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License.
 *
 * $Id: MetaDataHandlerV4.java,v 1.2 2000/07/20 01:28:18 pmahar Exp $
 */

package org.enhydra.kelp.common.bridge;

// Kelp imports
import org.enhydra.kelp.common.wizard.xmlc.Reporter;
import org.enhydra.kelp.common.node.OtterXMLCNode;
import org.enhydra.xml.xmlc.commands.xmlc.XMLCOptionsParser;

// XMLC imports
import org.enhydra.xml.xmlc.commands.options.OptionSet;
import org.enhydra.xml.xmlc.commands.options.OptionsParser;
import org.enhydra.xml.xmlc.XMLCException;
import org.enhydra.xml.xmlc.metadata.CompileOptions;
import org.enhydra.xml.xmlc.metadata.DocumentClass;
import org.enhydra.xml.xmlc.metadata.DOMEdits;
import org.enhydra.xml.xmlc.metadata.MetaData;
import org.enhydra.xml.xmlc.metadata.MetaDataDocument;

// Standard imports
import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;

public class MetaDataHandlerV4 implements MetaDataHandler {
  private MetaData metaData = null;
  private MetaDataDocument doc = null;

  public MetaDataHandlerV4() {
    doc = new MetaDataDocument();
    metaData = doc.getMetaData();
    metaData.setCompileOptions(new CompileOptions(doc));
    metaData.setDocumentClass(new DocumentClass(doc));
    metaData.setDOMEdits(new DOMEdits(doc));
  }

  // Compiler Options
  public boolean getCompileSource() {
    return metaData.getCompileOptions().getCompileSource();
  }

  public void setCompileSource(boolean b) {
    metaData.getCompileOptions().setCompileSource(b);
  }

  public String getDocumentOutput() {
     return metaData.getCompileOptions().getDocumentOutput();
  }

  public void setDocumentOutput(String s) {
    metaData.getCompileOptions().setDocumentOutput(s);
  }

  public String getInputDocument() {
    return metaData.getCompileOptions().getInputDocument();
  }

  public void setInputDocument(String s) {
    metaData.getCompileOptions().setInputDocument(s);
  }

  public boolean getKeepGeneratedSource() {
    return metaData.getCompileOptions().getKeepGeneratedSource();
  }

  public void setKeepGeneratedSource(boolean b) {
    metaData.getCompileOptions().setKeepGeneratedSource(b);
  }

  public boolean getPrintAccessorInfo() {
    return metaData.getCompileOptions().getPrintAccessorInfo();
  }

  public void setPrintAccessorInfo(boolean b) {
    metaData.getCompileOptions().setPrintAccessorInfo(b);
  }

  public boolean getPrintDOM() {
    return metaData.getCompileOptions().getPrintDOM();
  }
  public void setPrintDOM(boolean b) {
    metaData.getCompileOptions().setPrintDOM(b);
  }

  public boolean getPrintParseInfo() {
    return metaData.getCompileOptions().getPrintParseInfo();
  }

  public void setPrintParseInfo(boolean b) {
      metaData.getCompileOptions().setPrintParseInfo(b);
  }


  public boolean getPrintVersion() {
    return metaData.getCompileOptions().getPrintVersion();
  }

  public void setPrintVersion(boolean b) {
     metaData.getCompileOptions().setPrintVersion(b);
  }

  public boolean getVerbose() {
    return metaData.getCompileOptions().getVerbose();
  }

  public void setVerbose(boolean b) {
    metaData.getCompileOptions().setVerbose(b);
  }

  // Document Class
  public String getClassName() {
    return metaData.getDocumentClass().getName();
  }

  public void setClassName(String n) {
    metaData.getDocumentClass().setName(n);
  }

  public File getJavaClassSource() {
    return metaData.getDocumentClass().getJavaClassSource();
  }
  public void setJavaClassSource(File f, OtterXMLCNode node) {
    String outputRoot = node.getGenerateToRoot();
    String path       = f.getAbsolutePath();

    outputRoot = outputRoot.replace('\\','/').trim();
    path = path.replace('\\','/').trim();
    String name = pathToClass(outputRoot, path);
    metaData.getDocumentClass().setName(name);
    metaData.getCompileOptions().setClassOutputRoot(outputRoot);
    metaData.getCompileOptions().setSourceOutputRoot(outputRoot);
    try {
      doc.completeModifications();
    } catch (XMLCException e) {
      e.printStackTrace();
    }
  }


  public File getJavaInterfaceSource() {
    return metaData.getDocumentClass().getJavaInterfaceSource();
  }

  public void setJavaInterfaceSource(File f, OtterXMLCNode node) {
    // the rest should be automatic.
    try {
      doc.completeModifications();
    } catch (XMLCException e) {
      e.printStackTrace();
    }
 }


  public String getPackageName() {
    String name = null;
    try {
      name = metaData.getDocumentClass().getPackageName();
    } catch (NullPointerException e) {
      // Workaround?
      name = null;
    }
    return name;
  }

  public boolean getRecompilation() {
    return metaData.getDocumentClass().getRecompliation();
  }

  public void setRecompilation(boolean b) {
    metaData.getDocumentClass().setRecompliation(b);
  }

  // DOM Edits
  public Object[] getDeleteElements() {
    return metaData.getDOMEdits().getDeleteElements();
  }

  public Object[] getURLMappings() {
    return metaData.getDOMEdits().getURLMappings();
  }


  // Other
  public Object getMetaData() {
    return metaData;
  }

  public void parse(String[] files, String[] args, PrintWriter writer,
   OtterXMLCNode node)
     throws XMLCException, IOException {

    OptionsParser opParser  = null;
    OptionSet opSet = null;

    try {
      doc.completeModifications();

      opSet = new OptionSet();
      opSet.setMetaDataDoc(doc);
      opParser = new OptionsParser(opSet, new Reporter());
      if ((files.length > 0)) {
        opParser.parse(files);
       }
       if ((args.length > 0)) {
         opParser.parse(args);
       }
    } catch (XMLCException e) {
      e.printStackTrace();
      throw e;
    } catch (IOException e) {
      e.printStackTrace();
      throw e;
    } catch (Exception e) {
      if ((e instanceof XMLCException) ||
          (e instanceof IOException)) {
        // ignore
      } else {
        e.printStackTrace();
        throw new XMLCException(e);
      }
    }
  }


  public void save(File op) throws IOException {
    doc.serialize(op);
  }

  // Private Stuff
  private String pathToClass(String outputRoot, String in) {
    String out = in;
    int index = 0;

    if (out.startsWith(outputRoot)) {
      out = out.substring(outputRoot.length());
    }
    index = out.lastIndexOf(".java");
    if (index > -1) {
      out = out.substring(0, index);
    }
    out = out.replace('\\','.');
    out = out.replace('/','.');
    while (out.startsWith(".")) {
      out = out.substring(1);
    }
    while (out.endsWith(".")) {
      out = out.substring(0,out.length()-1);
    }

    return out;
  }

}