/*
 * Enhydra Java Application Server
 * The Initial Developer of the Original Code is Lutris Technologies Inc.
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc.
 * All Rights Reserved.
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License.
 *
 * $Id: Mapper.java,v 1.5 2000/07/26 02:50:25 pmahar Exp $
 */
package org.enhydra.kelp.common.node;
import java.io.File;

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 * @version $Revision: 1.5 $
 */
public class Mapper {
    private OtterProject project = null;
    private String[][]   map = new String[0][0];
    private String[]     path = new String[0];
    private int          scope = OtterProject.MAP_SCOPE_ALL;

    /**
     * Constructor declaration
     *
     *
     * @param p
     * @param f
     *
     * @see
     */
    public Mapper(OtterProject p) {
        project = p;
        scope = project.getMapScope();
        path = project.getSourcePathArray();
        if (scope != OtterProject.MAP_SCOPE_NONE) {
            map = project.getPackageMap();
        }
    }

    /**
     * Method declaration
     *
     *
     * @param node
     *
     * @return
     *
     * @see
     */
    public String getMappedOutputPath(OtterNode node) {
        StringBuffer buf = new StringBuffer();

        buf.append(project.getClassOutputPath());
        buf.append(File.separatorChar);
        buf.append(getMappedPath(node));
        return toCurrentSeparator(buf.toString());
    }

    /**
     * Method declaration
     *
     *
     * @param node
     *
     * @return
     *
     * @see
     */
    public String getMappedSourcePath(OtterNode node) {
        StringBuffer buf = new StringBuffer();

        buf.append(project.getSourcePathOf(node));
        buf.append(File.separatorChar);
        buf.append(getMappedPath(node));
        return toCurrentSeparator(buf.toString());
    }

    /**
     * Method declaration
     *
     *
     * @param node
     *
     * @return
     *
     * @see
     */
    private String getMappedPath(OtterNode node) {
        String sourcePath = null;
        String sourceFilePath = null;
        String mappedPath = null;

        sourcePath = project.getSourcePathOf(node);
        sourceFilePath = node.getFilePath();
        mappedPath =
            sourceFilePath.substring(sourcePath.length() + 1,
                                     sourceFilePath.lastIndexOf("."));
        mappedPath = getMappedName(mappedPath, sourceFilePath);
        mappedPath = mappedPath.replace('.', File.separatorChar);
        mappedPath =
            mappedPath
            + sourceFilePath.substring(sourceFilePath.lastIndexOf("."));
        mappedPath = toCurrentSeparator(mappedPath);
        return mappedPath;
    }

    /**
     * Method declaration
     *
     *
     * @param in
     *
     * @return
     *
     * @see
     */
    private String toCurrentSeparator(String in) {
        String out = null;

        out = in.replace('\\', File.separatorChar);
        out = out.replace('/', File.separatorChar);
        return out;
    }

    /**
     * Method declaration
     *
     *
     * @param node
     *
     * @return
     *
     * @see
     */
    public String getClassName(OtterXMLCNode node) {
        boolean custom = false;
        int     type = node.getClassNameType();
        String  sourcePath = project.getSourcePathOf(node);
        String  rawFilePath =
            (new File(node.getFilePath())).getAbsolutePath();
        String  mappedName = new String();

        if (sourcePath.length() > 0) {
            mappedName = rawFilePath.substring(sourcePath.length(),
                                               rawFilePath.lastIndexOf("."));
        } else {
            System.out.println("Warning: file not in source path: "
                               + rawFilePath);
            rawFilePath.substring(rawFilePath.lastIndexOf(File.separator)
                                  + 1, rawFilePath.lastIndexOf("."));
        }
        switch (type) {
        case OtterXMLCNode.CLASS_NAME_CUSTOM:
            if (node.getCustomClassName().trim().length() > 0) {
                mappedName = node.getCustomClassName();
                custom = true;
            }
            break;
        case OtterXMLCNode.CLASS_NAME_DEFAULT:

            // already at default.
            break;
        default:
            mappedName = getMappedName(mappedName, rawFilePath);
        }
        mappedName = validateClassName(mappedName, rawFilePath, type, custom);
        return mappedName;
    }

    /**
     * Method declaration
     *
     *
     * @param mappedName
     * @param rawFilePath
     * @param type
     * @param custom
     *
     * @return
     *
     * @see
     */
    public String validateClassName(String mappedName, String rawFilePath,
                                    int type, boolean custom) {
        String validName = mappedName.trim();

        validName = validName.replace('/', '.');
        validName = validName.replace('\\', '.');
        if (validName.startsWith(".")) {
            validName = validName.substring(1);
        }
        if (!custom) {
            int at = -1;

            if (validName.length() == 0) {

                // not in path, create packageless default name.
                at = rawFilePath.lastIndexOf(File.separatorChar);
                validName = rawFilePath.substring(at + 1);
                at = validName.lastIndexOf(".");
                validName = validName.substring(0, at);
            }
            at = rawFilePath.lastIndexOf(File.separatorChar);
            String baseName =
                rawFilePath.substring(at + 1, rawFilePath.lastIndexOf("."));

            at = validName.lastIndexOf(".");
            if (rawFilePath.endsWith(".wml")) {
                validName = validName.substring(0, at + 1) + baseName + "WML";
            } else {
                validName = validName.substring(0, at + 1) + baseName
                            + "HTML";
            }
        }
        return validName;
    }

    /**
     * Method declaration
     *
     *
     * @param mappedName
     * @param rawFilePath
     *
     * @return
     *
     * @see
     */
    private String getMappedName(String mappedName, String rawFilePath) {
        String   mappedPath = rawFilePath;
        String   matchPack = new String();
        String[] lowerFrom = null;
        String   lowerPath = null;
        String   lowerFile = null;
        String   toPackage = new String();
        boolean  useMap = false;
        boolean  mapped = false;
        int      max = -1;
        int      matchMax = -1;

        lowerFile = rawFilePath.toLowerCase();
        if (map != null) {
            for (int i = 0; i < map.length; i++) {

                lowerFrom = new String[1];
                lowerFrom[0] = map[i][0];
                for (int j = 0; j < lowerFrom.length; j++) {
                    lowerFrom[j] = lowerFrom[j].trim().toLowerCase();
                    if (lowerFrom[j].length() > max) {
                        max = lowerFrom[j].length();
                        toPackage = map[i][1].trim();
                        if (lowerFile.length() > lowerFrom[j].length()
                                && lowerFile.startsWith(lowerFrom[j])) {
                            matchPack = toPackage;
                            matchMax = max;
                            mapped = true;
                        }
                    }
                }
            }
        }
        int index = 0;

        if (mapped) {
            index = rawFilePath.lastIndexOf(".");
            mappedName = matchPack + rawFilePath.substring(matchMax, index);
        }
        return mappedName;
    }

}
