/*
 * Enhydra Java Application Server
 * The Initial Developer of the Original Code is Lutris Technologies Inc.
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc.
 * All Rights Reserved.
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License.
 *
 * $Id: CodeGenUtil.java,v 1.14 2000/07/26 10:01:30 pmahar Exp $
 */
package org.enhydra.kelp.common.wizard.codegen;

// Kelp imports
import org.enhydra.kelp.common.swing.ExtensionFilter;

import org.enhydra.kelp.common.node.Mapper;
import org.enhydra.kelp.common.node.OtterTemplateNode;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.OtterFolderNode;
import org.enhydra.kelp.common.node.OtterHTMLFileNode;
import org.enhydra.kelp.common.node.OtterJavaFileNode;
import org.enhydra.kelp.common.node.OtterResourceNode;
import org.enhydra.kelp.common.node.OtterTextFileNode;
import org.enhydra.kelp.common.node.OtterNodeFactory;
import org.enhydra.kelp.common.wizard.CodeImporter;
import org.enhydra.kelp.common.ValidationException;

// CodeGen imports
import org.enhydra.codegen.CodeGen;
import org.enhydra.codegen.DefaultOptionSet;
import org.enhydra.codegen.OptionSet;
import org.enhydra.codegen.Generator;
import org.enhydra.codegen.GeneratorException;
import org.enhydra.codegen.TemplateDrivenGenerator;
import org.enhydra.codegen.wizard.CodeGenDialog;
import org.enhydra.codegen.wizard.CodeGenPanel;
import org.enhydra.codegen.wizard.DefaultOptionPanel3;
import org.enhydra.codegen.webapp.WebAppGenerator;

// Standard imports
import java.awt.Component;
import java.io.File;
import java.io.FileInputStream;
import java.io.FilenameFilter;
import java.util.Enumeration;
import java.util.Properties;
import java.util.Vector;

/**
 * Class declaration
 *
 *
 * @author
 * @version %I%, %G%
 */
abstract public class CodeGenUtil {
    public final static int     CHOICE_CANCEL = CodeGenDialog.CHOICE_CANCEL;
    public final static int     CHOICE_FINISH = CodeGenDialog.CHOICE_FINISH;
    private final static String CODEGEN_CLASS = "org.enhydra.codegen.CodeGen";
    private OtterProject        project = null;
    private CodeGen             codeGen = null;
    private Mapper              mapper = null;
    private ExtensionFilter     optionFilter = null;
    private ExtensionFilter     javaFilter = null;
    private int                 choice = CodeGenUtil.CHOICE_CANCEL;

    /**
     * Constructor declaration
     *
     *
     * @see
     */
    public CodeGenUtil(Generator gen) throws CodeGenException {
        try {
          codeGen = new CodeGen(gen);
        } catch (GeneratorException e) {
            throw new CodeGenException(e);
        }
    }

    public CodeGenUtil(OtterProject project, Generator gen) throws CodeGenException {
        this(gen);
        init(project);
    }

    public void init(OtterProject proj) throws CodeGenException {
        this.project = proj;

        if (!CodeGenUtil.isCodeGenAvailable()) {
            throw new CodeGenException(CodeGenUtil.CODEGEN_CLASS
                                      + " not found in IDE class path");
        }
        try {
            optionFilter = new ExtensionFilter();
            optionFilter.setDirectoryValid(false);
            optionFilter.addExtension("xmlc");
            javaFilter = new ExtensionFilter();
            javaFilter.setDirectoryValid(false);
            javaFilter.addExtension("java");

            initOptions();
            initPanels();
        } catch (GeneratorException e) {
            throw new CodeGenException(e);
        }
    }


    public static Generator createGenerator() throws CodeGenException {
      throw new CodeGenException("CodeGenUtil.createGenerator()");
    }
    /**
     * Method declaration
     *
     *
     * @return
     *
     * @exception GeneratorException
     *
     * @see
     */
    public CodeGenPanel[] getWizardPanels() throws CodeGenException {
        CodeGenPanel[] panels = null;

        try {
            panels = getGenerator().getWizardPanels();
        } catch (GeneratorException e) {
            throw new CodeGenException(e);
        }
        return panels;
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public static String getDisplayName() {
        String displayName = "Web Application";

        if (CodeGenUtil.isCodeGenAvailable()) {
            displayName = WebAppGenerator.DISPLAY_NAME;
        }
        return displayName;
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public OtterProject getProject() {
        return project;
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @exception GeneratorException
     *
     * @see
     */
    public File[] generate() throws CodeGenException {
        File[] files = new File[0];

        try {
            for (int i = 0;
                    i < getGenerator().getWizardPanels().length;
                    i++) {
                getGenerator().getWizardPanels()[i].writeOptionSet();
            }
            files = getGenerator().generate();
        } catch (GeneratorException e) {
            throw new CodeGenException(e);
        }
        return files;
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public File getGeneratedSourceDir() {
        File                    pathFile = null;
        String                  path = new String();

        pathFile = getGenerator().getSourcePath();
        return pathFile;
    }

    public TemplateDrivenGenerator getGenerator() {
       TemplateDrivenGenerator generator = null;
       generator = (TemplateDrivenGenerator) codeGen.getSelection();
       return generator;
    }

    /**
     * Method declaration
     *
     *
     * @param owner
     *
     * @return
     *
     * @see
     */
    public File[] invokeWizard(Component owner) throws CodeGenException {
        File[] files = new File[0];

        files = codeGen.invokeWizard(owner);
        choice = codeGen.getChoice();
        if (choice == CodeGenUtil.CHOICE_FINISH && files != null) {
            setupProject(files);
        }
        return files;
    }

    /**
     * Constructor declaration
     *
     *
     * @param files
     *
     * @see
     */
    public void setupProject(File[] files) throws CodeGenException {
        CodeImporter importer = null;
        String pack = new String();
        String[][] map = new String[0][2];

        importer = new CodeImporter();
        try {
          importer.setRoot(new File(getGenerator().getProjectPath()));
          pack = getPackage();
          map = getGenerator().expandKelpPackageMap();
          importer.setPackage(pack);
          importer.setPackageMap(map);
          importer.setSource(getGeneratedSourceDir());
          importer.importFiles(project, files);
          String[] path = new String[1];
          path[0] = getGeneratedSourceDir().getAbsolutePath();
          project.setSourcePathArray(path);
        } catch (ValidationException e) {
            throw new CodeGenException(e);
        } catch (GeneratorException e) {
            throw new CodeGenException(e);
        }
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public int getChoice() {
        return choice;
    }



    /**
     * Method declaration
     *
     *
     *
     * @exception GeneratorException
     *
     * @see
     */
    private void initPanels() throws GeneratorException, CodeGenException {
        for (int i = 0; i < getWizardPanels().length; i++) {
            if (getWizardPanels()[i] instanceof DefaultOptionPanel3) {
                DefaultOptionPanel3 panel3 = null;

                panel3 = (DefaultOptionPanel3) getWizardPanels()[i];
                panel3.checkJBuilder.setSelected(false);
                panel3.checkJBuilder.setVisible(false);
                break;
            }
        }
    }

    /**
     * Method declaration
     *
     *
     * @param p
     *
     * @see
     */
    private void initOptions() throws GeneratorException {
        File   file = null;
        String dest = new String();
        String projectName = new String();
        String pack = new String();
        int    index = -1;

        dest = project.getCodeGenDefaultDestination();

        pack = project.getName();
        index = pack.indexOf(".");
        if (index > -1) {
            pack = pack.substring(0, index);
        }
        file = new File(project.getFilePath());
        if (! file.getParent().equalsIgnoreCase(dest)) {
          projectName = file.getParentFile().getName();
        }
        if (projectName.trim().length() == 0) {
          projectName = file.getName().substring(0,file.getName().indexOf('.'));
        }
        getGenerator().getOptionSet().lookup(DefaultOptionSet.DESTINATION).setValue(dest);
        getGenerator().getOptionSet().lookup(DefaultOptionSet.PROJECT).setValue(projectName);
        getGenerator().getOptionSet().lookup(DefaultOptionSet.PACKAGE).setValue(pack.toLowerCase());
        getGenerator().getOptionSet().lookup(DefaultOptionSet.MAKE).setValue(false);
        getGenerator().getOptionSet().lookup(DefaultOptionSet.SCRIPT).setValue(false);
        getGenerator().getOptionSet().lookup(DefaultOptionSet.JBUILDER).setValue(false);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @exception GeneratorException
     *
     * @see
     */
    private String getPackage() throws GeneratorException {
        OptionSet optionSet;
        String    pack;

        optionSet = getGenerator().getOptionSet();
        pack = optionSet.lookup(DefaultOptionSet.PACKAGE).getValue();
        return pack;
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public static boolean isCodeGenAvailable() {
        boolean available = true;

        try {
            Class testClass = Class.forName(CodeGenUtil.CODEGEN_CLASS);
        } catch (java.lang.ClassNotFoundException e) {
            available = false;
        }
        return available;
    }

}
