/*
 * Enhydra Java Application Server
 * The Initial Developer of the Original Code is Lutris Technologies Inc.
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc.
 * All Rights Reserved.
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License.
 *
 * $Id: ConfNodePropertyPanel.java,v 1.4 2000/07/21 19:53:03 pmahar Exp $
 */
package org.enhydra.kelp.common.wizard.conf;

import org.enhydra.kelp.common.wizard.AbstractPropertyPanel;
import org.enhydra.kelp.common.node.OtterTemplateNode;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.swing.SwingUtil;
import org.enhydra.kelp.common.swing.SwingTableSelectionEvent;
import org.enhydra.kelp.common.swing.SwingTableSelectionListener;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.Vector;

import javax.swing.border.TitledBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import javax.swing.table.AbstractTableModel;
import java.awt.*;
import javax.swing.*;
import java.beans.*;

public class ConfNodePropertyPanel extends JPanel
     implements ListSelectionListener, SwingTableSelectionListener {

  private OtterNode node = null;
  private GridBagLayout layoutMain;
  private JCheckBox checkConfigure;
  private JCheckBox checkOverwrite;
  private JPanel panelClassName;
  private GridBagLayout layoutClassName;
  private JPanel panelPath;
  private TitledBorder borderClassName;
  private LocalButtonListener buttonListener;
  private JTable	       tableConf;
  private JScrollPane	       scrollTable;
  private LocalTableModel      tableModel = new LocalTableModel();
  private JTextField textPath;
  private TitledBorder borderPath;
  private GridBagLayout layoutPath;
  private JButton buttonSet;
  private JButton buttonAdd;
  private JButton buttonEdit;
  private JButton buttonRemove;
  private JButton buttonDefault;
  private int currentSelectionIndex = -1;
  private Vector swingTableSelectionListeners = new Vector();
  private JButton buttonUp;
  private JButton buttonDown;
  private JCheckBox checkServer;

  public ConfNodePropertyPanel() {
    try {
      jbInit();
      pmInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

    /**
     * ListSelectionListener event
     */
    public void valueChanged(ListSelectionEvent e) {
	ListSelectionModel lsm = (ListSelectionModel) e.getSource();

	if (lsm.isSelectionEmpty()) {
	    currentSelectionIndex = -1;
	} else {
	    currentSelectionIndex = lsm.getMinSelectionIndex();
	}
        fireSwingTableSelectionEvent();
    }

    private void fireSwingTableSelectionEvent() {
       SwingTableSelectionListener listener = null;
       for (int i = 0 ; i < swingTableSelectionListeners.size() ; i++ ) {
         listener = (SwingTableSelectionListener) swingTableSelectionListeners.elementAt(i);
         listener.onSwingTableSelection(
            new SwingTableSelectionEvent(this, currentSelectionIndex));
       }
    }

  public synchronized void addSwingTableSelectionListener(SwingTableSelectionListener l) {
    if (! swingTableSelectionListeners.contains(l)) {
       swingTableSelectionListeners.addElement(l);
    }
  }

  public synchronized void removeSwingTableSelectionListener(SwingTableSelectionListener l) {
    if (swingTableSelectionListeners.contains(l)) {
      swingTableSelectionListeners.removeElement(l);
    }
  }



    public void onSwingTableSelection(SwingTableSelectionEvent event) {
      boolean selection = (! event.isSelectionNull());
      buttonEdit.setEnabled(selection);
      buttonRemove.setEnabled(selection);
      if (selection) {
        int index = event.getSelectionIndex();
        boolean down = (index < tableModel.getRowCount() -1);
        boolean up = (index > 0);
        buttonUp.setEnabled(up);
        buttonDown.setEnabled(down);
      } else {
        buttonUp.setEnabled(false);
        buttonDown.setEnabled(false);
      }
    }

  public OtterNode getNode() {
    return node;
  }

  public void setNode(OtterNode n) {
    node = n;
  }


  private void pmInit() throws Exception {
    buttonEdit.setEnabled(false);
    buttonEdit.setToolTipText("Edit selected row");
    buttonRemove.setEnabled(false);
    buttonRemove.setToolTipText("Remove selected row");
    buttonListener = new LocalButtonListener();
    buttonSet.addActionListener(buttonListener);
    buttonAdd.addActionListener(buttonListener);
    buttonAdd.setToolTipText("Add a new row");
    buttonEdit.addActionListener(buttonListener);
    buttonDefault.addActionListener(buttonListener);
    buttonDefault.setToolTipText("Reset to default");
    buttonRemove.addActionListener(buttonListener);
    buttonUp.addActionListener(buttonListener);
    buttonDown.addActionListener(buttonListener);
    tableModel = new LocalTableModel();
    tableConf.setModel(tableModel);
//	tableConf.setDefaultRenderer(String.class, new CellRendererWithToolTip());
	tableModel.addTableModelListener(tableConf);
	tableConf.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
	tableConf.getTableHeader().setUpdateTableInRealTime(false);
	tableConf.getTableHeader().setReorderingAllowed(false);
	ListSelectionModel rowSM = tableConf.getSelectionModel();

	rowSM.addListSelectionListener(this);
        addSwingTableSelectionListener(this);

  }

  private void jbInit() throws Exception {
    checkConfigure = (JCheckBox) Beans.instantiate(getClass().getClassLoader(), JCheckBox.class.getName());
    checkOverwrite    = (JCheckBox) Beans.instantiate(getClass().getClassLoader(), JCheckBox.class.getName());
    checkServer    = (JCheckBox) Beans.instantiate(getClass().getClassLoader(), JCheckBox.class.getName());

    layoutMain = (GridBagLayout) Beans.instantiate(getClass().getClassLoader(), GridBagLayout.class.getName());
    layoutPath = (GridBagLayout) Beans.instantiate(getClass().getClassLoader(), GridBagLayout.class.getName());

    panelClassName = (JPanel) Beans.instantiate(getClass().getClassLoader(), JPanel.class.getName());
    layoutClassName = (GridBagLayout) Beans.instantiate(getClass().getClassLoader(), GridBagLayout.class.getName());
    panelPath = (JPanel) Beans.instantiate(getClass().getClassLoader(), JPanel.class.getName());
      buttonSet = (JButton) Beans.instantiate(getClass().getClassLoader(), JButton.class.getName());
      textPath = (JTextField) Beans.instantiate(getClass().getClassLoader(), JTextField.class.getName());


    tableConf = (JTable) Beans.instantiate(getClass().getClassLoader(), JTable.class.getName());
    buttonAdd = (JButton) Beans.instantiate(getClass().getClassLoader(), JButton.class.getName());
    buttonEdit = (JButton) Beans.instantiate(getClass().getClassLoader(), JButton.class.getName());
    buttonRemove = (JButton) Beans.instantiate(getClass().getClassLoader(), JButton.class.getName());
    buttonDefault = (JButton) Beans.instantiate(getClass().getClassLoader(), JButton.class.getName());
    buttonUp = (JButton) Beans.instantiate(getClass().getClassLoader(), JButton.class.getName());
    buttonDown = (JButton) Beans.instantiate(getClass().getClassLoader(), JButton.class.getName());
    checkConfigure.setText("Generate .conf");
    checkConfigure.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        enableUI(checkConfigure.isSelected());
      }
    });

    checkOverwrite.setText("Overwrite without warning");
    checkServer.setText("Use this file to configure servlet server");

    panelClassName.setLayout(layoutClassName);
    borderClassName = BorderFactory.createTitledBorder("Replace text");
    panelClassName.setBorder(borderClassName);

    scrollTable = new JScrollPane(tableConf);
    tableConf.setToolTipText(new String());
    tableConf.sizeColumnsToFit(JTable.AUTO_RESIZE_ALL_COLUMNS);
    tableConf.setAutoResizeMode(JTable.AUTO_RESIZE_LAST_COLUMN);
    tableConf.setColumnSelectionAllowed(false);
    scrollTable.setMinimumSize(new Dimension(300, 100));
    scrollTable.setPreferredSize(new Dimension(300, 100));
    buttonAdd.setText("Add..");
    buttonEdit.setText("Edit...");
    buttonRemove.setText("Remove...");
    buttonDefault.setText("Default...");
    buttonUp.setToolTipText("Move before previous row");
    buttonUp.setText("Up");
    buttonDown.setToolTipText("Move after next row");
    buttonDown.setText("Down");
    scrollTable.getViewport().add(tableConf, BorderLayout.CENTER);


    panelClassName.add(scrollTable, new GridBagConstraints(3, 0, 6, 1, 0.8, 0.8
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
    panelClassName.add(buttonAdd, new GridBagConstraints(3, 1, 1, 1, 0.1, 0.1
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(2, 1, 2, 1), 0, 0));
    panelClassName.add(buttonEdit, new GridBagConstraints(4, 1, 1, 1, 0.1, 0.1
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(2, 1, 2, 1), 0, 0));
    panelClassName.add(buttonRemove, new GridBagConstraints(5, 1, 1, 1, 0.1, 0.1
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(2, 1, 2, 1), 0, 0));
    panelClassName.add(buttonDefault, new GridBagConstraints(8, 1, 1, 1, 0.1, 0.1
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(2, 1, 2, 1), 0, 0));
    panelClassName.add(buttonUp, new GridBagConstraints(6, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(2, 1, 2, 1), 0, 0));
    panelClassName.add(buttonDown, new GridBagConstraints(7, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(2, 1, 2, 1), 0, 0));


      borderPath = BorderFactory.createTitledBorder("Location for servlet configuration files");
      textPath.setMinimumSize(new Dimension(100, 21));
    textPath.setPreferredSize(new Dimension(100, 21));
    textPath.setEditable(true);
      buttonSet.setText("Set...");

      panelPath.setLayout(layoutPath);
      panelPath.setBorder(borderPath);
      panelPath.add(textPath, new GridBagConstraints(0, 0, 1, 1, 1.0, 0.1
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 5, 5), 100, 0));
      panelPath.add(buttonSet, new GridBagConstraints(1, 0, 1, 1, 0.1, 0.1
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));


    this.setLayout(layoutMain);
    this.add(checkConfigure, new GridBagConstraints(0, 0, 1, 1, 0.1, 0.1
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 5, 5), 0, 0));
    this.add(checkOverwrite, new GridBagConstraints(0, 1, 1, 1, 0.1, 0.1
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(5, 20, 5, 5), 0, 0));
    this.add(checkServer, new GridBagConstraints(0, 2, 1, 1, 0.1, 0.1
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(5, 20, 5, 5), 0, 0));
    this.add(panelClassName, new GridBagConstraints(0, 3, 1, 1, 0.1, 0.1
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(3, 15, 3, 5), 0, 0));
    this.add(panelPath, new GridBagConstraints(0, 4, 1, 1, 0.1, 0.1
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(3, 15, 5, 5), 0, 0));
  }

  public void readProperties() {
    String stringIn = null;
    File   fileIn   = null;
    OtterProject project = node.getProject();
    if (node instanceof OtterTemplateNode) {
      OtterTemplateNode confNode = (OtterTemplateNode) node;
      checkConfigure.setSelected(confNode.isSelected());
      checkServer.setSelected(confNode.isServer());
      enableUI(checkConfigure.isSelected());
    } else {
      checkConfigure.setVisible(false);
      checkServer.setVisible(false);
      enableUI(true);
    }
    checkOverwrite.setSelected(project.isConfOverwriteSelected());
    putConfTable(project.getConfTable());
    textPath.setText(project.getServletConfPath());
  }

  public void writeProperties() {
    OtterProject project = node.getProject();
    if (node instanceof OtterTemplateNode) {
      OtterTemplateNode confNode = (OtterTemplateNode) node;
      confNode.setSelected(checkConfigure.isSelected());
      confNode.setServer(checkServer.isSelected());
    }
    project.setConfOverwriteSelected(checkOverwrite.isSelected());
    project.setConfTable(getConfTable());
    project.setConfPath(textPath.getText());
  }

  private String[][] getConfTable() {
    String[][] confTable = null;
    LocalRow   row       = null;
    int length = tableModel.getRowCount();
    confTable = new String[length][2];
    for (int i = 0 ; i < length ; i++ ) {
      row = tableModel.getRow(i);
      confTable[i][0] = row.getFind();
      confTable[i][1] = row.getReplaceWith();
    }
    return confTable;
  }

  private void putConfTable(String[][] confTable) {
    int length = tableModel.getRowCount();
    tableModel.removeAllRows();
    for (int i = 0 ; i < confTable.length ; i++) {
      tableModel.addRow(confTable[i][0], confTable[i][1]);
    }
    removeTableSelections();
  }

  private void removeTableSelections() {
    if (tableConf.getSelectedRow() > -1) {
      tableConf.removeRowSelectionInterval(
          tableConf.getSelectedRow(), tableConf.getSelectedRow());
    }
    currentSelectionIndex = -1;
    fireSwingTableSelectionEvent();
    tableConf.updateUI();
  }


  private void enableUI(boolean enable) {
    checkOverwrite.setEnabled(enable);
    tableConf.setEnabled(enable);
    textPath.setEnabled(enable);
    buttonAdd.setEnabled(enable);
    buttonEdit.setEnabled(currentSelectionIndex > -1);
    buttonRemove.setEnabled(currentSelectionIndex > -1);
    buttonDefault.setEnabled(enable);
    buttonSet.setEnabled(enable);
  }

    private void browseForPath() {
       File d = new File(textPath.getText());
       d = SwingUtil.getDirectoryChoice(this, d,
  	  "Select directory in which to create conf files");
       if (d != null && d.exists() && d.isDirectory()) {
         textPath.setText(d.getAbsolutePath());
       }
    }

    private void defaultTable() {
      int result = JOptionPane.YES_OPTION;
      if (tableModel.getRowCount() > 0) {
        result = JOptionPane.showConfirmDialog(this,
          "Replace current set with default?",
          "Switch to Default", JOptionPane.YES_NO_OPTION);
      }
      if (result == JOptionPane.YES_OPTION) {
         OtterProject project = node.getProject();
         putConfTable(project.getDefaultConfTable());
      }
    }

    private void addRow() {
       ReplaceEditorDialog d = null;
        if (getTopLevelAncestor() instanceof JDialog) {
          JDialog parentDialog = (JDialog)this.getTopLevelAncestor();
	  d = new ReplaceEditorDialog(parentDialog,
                "Add Replace Text Definition",	 true);
          d.setFind(new String());
          d.setReplaceWith(new String());
  	  d.show();

	  if (d.getOption() == ReplaceEditorDialog.ACTION_OK) {
 	      if (d.getFind().trim().length() > 0) {
                tableModel.addRow(d.getFind(), d.getReplaceWith());
                removeTableSelections();
  	      }
   	  }
      }

    }

    private void removeCurrent() {
      int result = 0;
      if (currentSelectionIndex > -1 &&
          currentSelectionIndex < tableModel.getRowCount()) {
        result = JOptionPane.showConfirmDialog(this,
          "Remove current replacement?",
          "Confirm Remove", JOptionPane.YES_NO_OPTION);
        if (result == JOptionPane.YES_OPTION) {
          tableModel.removeRow(currentSelectionIndex);
          removeTableSelections();
        }
      }
    }

    private void editCurrent() {
	ReplaceEditorDialog d = null;
        LocalRow row = tableModel.getRow(currentSelectionIndex);
        if (getTopLevelAncestor() instanceof JDialog) {
          JDialog parentDialog = (JDialog)this.getTopLevelAncestor();
	  d = new ReplaceEditorDialog(parentDialog,
                "Edit Replace Text Definition",	 true);
          d.setFind(row.getFind());
          d.setReplaceWith(row.getReplaceWith());
  	  d.show();

	  if (d.getOption() == ReplaceEditorDialog.ACTION_OK) {
 	      if (d.getFind().trim().length() > 0) {
                row.setFind(d.getFind());
                row.setReplaceWith(d.getReplaceWith());
                tableConf.updateUI();
  	      }
   	  }
      }
    }

    private void moveRowUp() {
      tableModel.moveUp(currentSelectionIndex);
      tableConf.updateUI();
    }

    private void moveRowDown() {
      tableModel.moveDown(currentSelectionIndex);
      tableConf.updateUI();
    }


    private class LocalButtonListener implements ActionListener {
      public void actionPerformed(ActionEvent e) {
        Object source = e.getSource();
        if (source == buttonSet) {
          browseForPath();
        } else if (source == buttonDefault) {
          defaultTable();
        } else if (source == buttonRemove) {
          removeCurrent();
        } else if (source == buttonAdd) {
          addRow();
        } else if (source == buttonEdit) {
          editCurrent();
        } else if (source == buttonUp) {
          moveRowUp();
        } else if (source == buttonDown) {
          moveRowDown();
        }
      }
    }

    /**
     * Class declaration
     *
     *
     * @author
     * @version %I%, %G%
     */
    class LocalTableModel extends AbstractTableModel {
	private Vector rowVector = new Vector();

	/**
	 * Constructor declaration
	 *
	 *
	 * @see
	 */
	public LocalTableModel() {}

	/**
	 * Method declaration
	 *
	 *
	 * @return
	 *
	 * @see
	 */
	public int getRowCount() {
	    int count = 0;

	    if (rowVector != null) {
		count = rowVector.size();
	    }

	    return count;
	}

	/**
	 * Method declaration
	 *
	 *
	 * @return
	 *
	 * @see
	 */
	public int getColumnCount() {
	    return 2;
	}

	/**
	 * Method declaration
	 *
	 *
	 * @param columnIndex
	 *
	 * @return
	 *
	 * @see
	 */
	public String getColumnName(int columnIndex) {
	    String name = new String();

	    switch (columnIndex) {

	    case 0:
		name = "Text to find";

		break;

	    case 1:
		name = "Replace with";

		break;
	    }

	    return name;
	}

	/**
	 * Method declaration
	 *
	 *
	 * @param columnIndex
	 *
	 * @return
	 *
	 * @see
	 */
	public Class getColumnClass(int columnIndex) {
	    Class  columnClass = null;
	    Object value = getValueAt(0, columnIndex);

	    if (value != null) {
		columnClass = value.getClass();
	    }

	    return columnClass;
	}

	/**
	 * Method declaration
	 *
	 *
	 * @param rowIndex
	 * @param columnIndex
	 *
	 * @return
	 *
	 * @see
	 */
	public boolean isCellEditable(int rowIndex, int columnIndex) {
	    return false;
	}

	/**
	 * Method declaration
	 *
	 *
	 * @param rowIndex
	 * @param columnIndex
	 *
	 * @return
	 *
	 * @see
	 */
	public Object getValueAt(int rowIndex, int columnIndex) {
	    Object value = null;

	    if (!isTableEmpty()) {
		LocalRow  row = (LocalRow) rowVector.elementAt(rowIndex);
		LocalCell cell = new LocalCell(columnIndex, row);

		value = cell.getValue();
	    }

	    return value;
	}

	/**
	 * Method declaration
	 *
	 *
	 * @param aValue
	 * @param rowIndex
	 * @param columnIndex
	 *
	 * @see
	 */
	public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
	    if (!isTableEmpty()) {
		LocalRow  row = (LocalRow) rowVector.elementAt(rowIndex);
		LocalCell cell = new LocalCell(columnIndex, row);

		cell.setValue(aValue);
		fireTableCellUpdated(columnIndex, rowIndex);
	    }
	}

	/**
	 * Method declaration
	 *
	 *
	 * @param rowIndex
	 *
	 * @return
	 *
	 * @see
	 */
	protected LocalRow getRow(int rowIndex) {
	    LocalRow row = null;

	    if (!isTableEmpty()) {
		if (rowVector.size() > rowIndex) {
		    row = (LocalRow) rowVector.elementAt(rowIndex);
		}
	    }

	    return row;
	}

	/**
	 * Method declaration
	 *
	 *
	 * @return
	 *
	 * @see
	 */
	protected Object[] getWideValues() {
	    Object[] wideValues = null;
	    String   stringValue = " ";
	    String   current = " ";
	    int      rowCount = 0;

	    rowCount = getRowCount();
	    wideValues = new Object[2];
	    wideValues[0] = "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXX";
	    wideValues[1] = "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXX";

	    for (int column = 1; column < wideValues.length; column++) {
		for (int row = 0; row < rowCount; row++) {
		    stringValue = (String) getValueAt(row, column);
		    stringValue = stringValue.trim();
		    current = (String) wideValues[column];

		    if (current == null) {
			wideValues[column] = stringValue;
		    } else {
			if (stringValue.trim().length() > current.length()) {
			    wideValues[column] = stringValue;
			}
		    }
		}
	    }

	    return wideValues;
	}

	/**
	 * Method declaration
	 *
	 *
	 * @see
	 */
	protected void saveMap() {
            OtterProject project = node.getProject();
	    int	       rowCount = rowVector.size();
	    String[][] map = new String[rowCount][2];
	    LocalRow   row = null;

	    for (int i = 0; i < rowCount; i++) {
		row = (LocalRow) rowVector.elementAt(i);
		map[i][0] = row.getFind();
		map[i][1] = row.getReplaceWith();
	    }

	    project.setPackageMap(map);
	}

	/**
	 * Method declaration
	 *
	 *
	 * @see
	 */
	protected void populateModel() {
            OtterProject project = node.getProject();
	    String[][] map = project.getPackageMap();

	    if (map != null) {
		int rowCount = map.length;

		for (int i = 0; i < rowCount; i++) {
		    addRow(map[i][0], map[i][1]);
		}
	    }
	}

	// /
	// /

	/**
	 * Method declaration
	 *
	 *
	 * @return
	 *
	 * @see
	 */
	private boolean isTableEmpty() {
	    boolean empty = true;

	    if (rowVector != null) {
		if (rowVector.size() > 0) {
		    empty = false;
		}
	    }

	    return empty;
	}

	/**
	 * Method declaration
	 *
	 *
	 * @param f
	 * @param p
	 *
	 * @see
	 */
	private void addRow(String f, String p) {
	    LocalRow newRow = null;

	    newRow = new LocalRow(f, p);

	    rowVector.addElement(newRow);

	    int size = rowVector.size();

	    fireTableDataChanged();
	}

	/**
	 * Method declaration
	 *
	 *
	 * @param index
	 *
	 * @see
	 */
	private void removeRow(int index) {
            if (index < rowVector.size()) {
  	      rowVector.removeElementAt(index);
	      fireTableDataChanged();
            }
	}

       private void moveUp(int index) {
         if (index > 0 && index < rowVector.size()) {
            Object current = rowVector.elementAt(index);
            Object previous = rowVector.elementAt(index - 1);
            rowVector.setElementAt(current, index - 1);
            rowVector.setElementAt(previous, index);
 	    fireTableDataChanged();
          }
       }

       private void moveDown(int index) {
         if (index < (rowVector.size() - 1)) {
            Object current = rowVector.elementAt(index);
            Object next    = rowVector.elementAt(index + 1);
            rowVector.setElementAt(current, index + 1);
            rowVector.setElementAt(next, index);
 	    fireTableDataChanged();
          }
       }

       private void removeAllRows() {
         rowVector.removeAllElements();
         fireTableDataChanged();
       }


    }

    /**
     * Class declaration
     *
     *
     * @author
     * @version %I%, %G%
     */
    class LocalRow {
	private String find = new String();
	private String replaceWith = new String();

	/**
	 * Constructor declaration
	 *
	 *
	 * @param f
	 * @param p
	 *
	 * @see
	 */
	public LocalRow(String f, String rw) {
	    find = f;
	    replaceWith = rw;
	}

	/**
	 * Method declaration
	 *
	 *
	 * @return
	 *
	 * @see
	 */
	public String getFind() {
	    return find;
	}

	/**
	 * Method declaration
	 *
	 *
	 * @param f
	 *
	 * @see
	 */
	public void setFind(String f) {
	    find = f;

	    saveToProject();
	}

	/**
	 * Method declaration
	 *
	 *
	 * @return
	 *
	 * @see
	 */
	public String getReplaceWith() {
	    return replaceWith;
	}

	/**
	 * Method declaration
	 *
	 *
	 * @param p
	 *
	 * @see
	 */
	public void setReplaceWith(String rw) {
	    replaceWith = rw;

	    saveToProject();
	}

	/**
	 * Method declaration
	 *
	 *
	 * @see
	 */
	private void saveToProject() {}

    }

    /**
     * Class declaration
     *
     *
     * @author
     * @version %I%, %G%
     */
    class LocalCell {
	private LocalRow row;
	private int    column;

	/**
	 * Constructor declaration
	 *
	 *
	 * @param c
	 * @param r
	 *
	 * @see
	 */
	public LocalCell(int c, LocalRow r) {
	    column = c;
	    row = r;
	}

	/**
	 * Method declaration
	 *
	 *
	 * @param value
	 *
	 * @return
	 *
	 * @see
	 */
	public boolean setValue(Object value) {
	    boolean set = true;

	    switch (column) {

	    case 0:
		row.setFind((String) value);

		break;

	    case 1:
		row.setReplaceWith((String) value);

		break;

	    default:
		set = false;

		break;
	    }

	    return set;
	}

	/**
	 * Method declaration
	 *
	 *
	 * @return
	 *
	 * @see
	 */
	public Object getValue() {
	    Object value = null;

	    switch (column) {

	    case 0:
		value = row.getFind();

		break;

	    case 1:
		value = row.getReplaceWith();

		break;
	    }

	    return value;
	}

	// /
	// /

	/**
	 * Method declaration
	 *
	 *
	 * @return
	 *
	 * @see
	 */
	protected LocalRow getRow() {
	    return row;
	}

    }
}