
/**
 * Title:        Kelp
 * Description:  Enhydra tools for Java IDEs
 * Copyright:    Copyright (c) 1999-2000
 * Company:      Lutris
 * @author Lutris Technologies
 * @version
 */

package org.enhydra.kelp.common.wizard.xmlc;

// Kelp imports
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.OtterXMLCNode;

// standard imports
import java.awt.*;
import javax.swing.*;
import java.beans.*;

public class XMLCOptionTab extends JPanel {
  private BorderLayout borderLayout1 = new BorderLayout();
  private JTabbedPane tab;
  private OtterNode node = null;
  private XMLCOptionPanel filePanel;
  private XMLCOptionPanel packagePanel;
  private XMLCOptionPanel projectPanel;

  public XMLCOptionTab() {
    try {
      jbInit();
      pmInit();
    }
    catch(Exception ex) {
      ex.printStackTrace();
    }
  }

  public void setNode(OtterNode n) {
    node = n;
 }


 public OtterNode getNode() {
   return node;
 }

 public void readProperties() {
   filePanel.readProperties();
   packagePanel.readProperties();
   projectPanel.readProperties();
 }

 public void writeProperties() {
    filePanel.writeProperties();
    packagePanel.writeProperties();
    projectPanel.writeProperties();
 }

 public void setEnabled(boolean b) {
   super.setEnabled(b);
   filePanel.setEnabled(b);
   packagePanel.setEnabled(b);
   projectPanel.setEnabled(true);
 }

 public void init() {
   OtterNode parent = null;
   if (node != null) {
     projectPanel.setNode(node.getProject());
     if (node instanceof OtterXMLCNode) {
       filePanel.setNode(node);
       parent = node.getParent();
       if (parent == null || parent instanceof OtterProject) {
         tab.remove(packagePanel);
       } else {
         packagePanel.setNode(node.getParent());
       }
     } else {
       packagePanel.setNode(node);
       tab.remove(filePanel);
     }
   }
 }

  private void pmInit() {
    setBorder(filePanel.getBorder());
    filePanel.setBorder(null);
    packagePanel.setBorder(null);
    projectPanel.setBorder(null);
  }

  private void jbInit() throws Exception {
    tab = (JTabbedPane) Beans.instantiate(getClass().getClassLoader(), JTabbedPane.class.getName());
    filePanel = (XMLCOptionPanel) Beans.instantiate(getClass().getClassLoader(), XMLCOptionPanel.class.getName());
    packagePanel = (XMLCOptionPanel) Beans.instantiate(getClass().getClassLoader(), XMLCOptionPanel.class.getName());
    projectPanel = (XMLCOptionPanel) Beans.instantiate(getClass().getClassLoader(), XMLCOptionPanel.class.getName());
    this.setLayout(borderLayout1);
    this.add(tab, BorderLayout.CENTER);
    tab.add(filePanel,  "File");
    tab.add(packagePanel, "Package/Folder");
    tab.add(projectPanel, "Project");
  }
}