/*
 * Enhydra Java Application Server
 * The Initial Developer of the Original Code is Lutris Technologies Inc.
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc.
 * All Rights Reserved.
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License.
 *
 * $Id: PrimeNode.java,v 1.3 2000/08/02 22:48:28 pmahar Exp $
 */

package org.enhydra.kelp.jbuilder.node;

import org.enhydra.kelp.common.PropUtil;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.node.OtterProject;
import com.borland.jbuilder.node.JBProject;
import com.borland.primetime.node.FileNode;
import com.borland.primetime.node.Node;
import com.borland.primetime.vfs.Url;
import java.io.File;

/**
 * Class declaration
 *
 *
 * @author
 * @version %I%, %G%
 */
public class PrimeNode implements OtterNode {
    private Node nativeNode = null;

    /**
     * Constructor declaration
     *
     *
     * @see
     */
    public PrimeNode() {}

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public char getFileSeparatorChar() {
	return '/';
    }

    /**
     * Constructor declaration
     *
     *
     * @param node
     *
     * @see
     */
    public PrimeNode(Node n) {
	nativeNode = n;
    }

    public OtterProject getProject() {
      return (new PrimeProject((JBProject)nativeNode.getProject()));
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String getFilePath() {
	String out = null;

	if (nativeNode != null) {
	    if (nativeNode instanceof FileNode) {
		FileNode fileNode;

		fileNode = (FileNode) nativeNode;
		out = fileNode.getUrl().getFileObject().getAbsolutePath();
	    }
	}

	return out;
    }

    public String getXMLCOptionFilename() {
      String filename = new String();
      File file = null;
      file = PropUtil.getFileRelativeToProject(this,
                       OtterNode.XMLC_OPTION_FILENAME);
      if (file != null &&
          file.isFile() &&
          file.getAbsolutePath().toLowerCase().endsWith(".xmlc")) {
          filename = file.getAbsolutePath();
      }
      return filename;
    }

    public void setXMLCOptionFilename(String n) {
       File   file = null;
       String relFilename = new String();
       file = new File(n);
       if (file != null &&
           file.isFile() &&
           file.getAbsolutePath().toLowerCase().endsWith(".xmlc")) {
         relFilename = PropUtil.compressPathRelativeToProject(this, n);
       }
       setProperty(OtterNode.XMLC_OPTION_FILENAME, relFilename);
    }

    public String getXMLCParameters() {
      return getProperty(OtterNode.XMLC_PARAMETERS);
    }

    public void setXMLCParameters(String p) {
      setProperty(OtterNode.XMLC_PARAMETERS, p);
    }



    /**
     * Method declaration
     *
     *
     * @param property
     *
     * @return
     *
     * @see
     */
    public String getProperty(String property) {
	String out = null;

	if (nativeNode != null) {
	    out = nativeNode.getProperty(property);
	}

	return out;
    }

    /**
     * Method declaration
     *
     *
     * @param property
     * @param value
     *
     * @see
     */
    public void setProperty(String property, String value) {
	if (nativeNode != null) {
	    nativeNode.setProperty(property, value);
	}
    }

    public void setProperty(String property, int i) {
        String value = Integer.toString(i);
	setProperty(property, value);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public Object getNativeNode() {
	return nativeNode;
    }

    /**
     * Method declaration
     *
     *
     * @param n
     *
     * @see
     */
    public void setNativeNode(Object n) {
	nativeNode = (Node) n;
    }

    public OtterNode getParent() {
      OtterNode parent = null;
      Node nativeParent = null;
      nativeParent = nativeNode.getParent();
      if (nativeParent instanceof JBProject) {
        parent = getProject();
      } else {
        parent = new PrimeNode(nativeParent);
      }
      return parent;
    }

}



