/*
 * Enhydra Java Application Server
 * The Initial Developer of the Original Code is Lutris Technologies Inc.
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc.
 * All Rights Reserved.
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License.
 *
 * $Id: XMLCNodePropertyPanel.java,v 1.6 2000/07/24 07:24:41 pmahar Exp $
 */
package org.enhydra.kelp.jbuilder.properties;

// Kelp imports
import org.enhydra.kelp.common.node.OtterXMLCNode;
import org.enhydra.kelp.common.wizard.xmlc.MapDialog;
import org.enhydra.kelp.common.wizard.xmlc.XMLCOptionTab;

// Standard imports
import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.io.File;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 * @version $Revision: 1.6 $
 */
public class XMLCNodePropertyPanel extends JPanel {
    private OtterXMLCNode       node;
    private GridBagLayout       layoutMain;
    private JCheckBox           checkCompile;
    private JCheckBox           checkKeep;
    private JTextField          textClassName;
    private JPanel              panelClassName;
    private GridBagLayout       layoutClassName;
    private JRadioButton        radioDefault;
    private JRadioButton        radioCustom;
    private JRadioButton        radioMap;
    private JButton             buttonMap;
    private XMLCOptionTab       optionTab;
    private TitledBorder        borderClassName;
    private ButtonGroup         radioGroup;
    private JCheckBox           checkStatic;
    private LocalButtonListener buttonListener;
    private LocalCheckListener  checkListener;
    private LocalRadioListener  radioListener;

    /**
     * Constructor declaration
     *
     *
     * @see
     */
    public XMLCNodePropertyPanel() {
        try {
            jbInit();
            pmInit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public OtterXMLCNode getNode() {
        return node;
    }

    /**
     * Method declaration
     *
     *
     * @param n
     *
     * @see
     */
    public void setNode(OtterXMLCNode n) {
        node = n;
    }

    /**
     * Method declaration
     *
     *
     * @exception Exception
     *
     * @see
     */
    private void pmInit() throws Exception {
        radioGroup =
            (ButtonGroup) Beans.instantiate(getClass().getClassLoader(),
                                            ButtonGroup.class.getName());
        radioGroup.add(radioDefault);
        radioGroup.add(radioCustom);
        radioGroup.add(radioMap);
        radioSelection();
        radioListener  = new LocalRadioListener();
        radioDefault.addChangeListener(radioListener);
        radioCustom.addChangeListener(radioListener);
        radioMap.addChangeListener(radioListener);
        //
        buttonListener = new LocalButtonListener();
        buttonMap.addActionListener(buttonListener);
        //
        checkListener = new LocalCheckListener();
        checkCompile.addActionListener(checkListener);
        checkStatic.addActionListener(checkListener);
    }

    /**
     * Method declaration
     *
     *
     * @exception Exception
     *
     * @see
     */
    private void jbInit() throws Exception {
        checkCompile =
            (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                          JCheckBox.class.getName());
        checkKeep = (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                                  JCheckBox.class.getName());
        textClassName =
            (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                           JTextField.class.getName());
        layoutMain =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        panelClassName =
            (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                       JPanel.class.getName());
        layoutClassName =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        radioDefault =
            (JRadioButton) Beans.instantiate(getClass().getClassLoader(),
                                             JRadioButton.class.getName());
        radioCustom =
            (JRadioButton) Beans.instantiate(getClass().getClassLoader(),
                                             JRadioButton.class.getName());
        radioMap =
            (JRadioButton) Beans.instantiate(getClass().getClassLoader(),
                                             JRadioButton.class.getName());
        buttonMap = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                                JButton.class.getName());
        optionTab =
            (XMLCOptionTab) Beans.instantiate(getClass().getClassLoader(),
                                                XMLCOptionTab.class.getName());
        checkStatic =
            (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                          JCheckBox.class.getName());
        checkCompile.setText("Generate DOM class");
        checkKeep.setText("Keep generated Java source");
        optionTab.setLayout(optionTab.getLayout());
        panelClassName.setLayout(layoutClassName);
        borderClassName =
            BorderFactory.createTitledBorder("Generated class name");
        panelClassName.setBorder(borderClassName);
        radioDefault.setSelected(true);
        radioDefault.setText("Default");
        radioCustom.setText("Custom");
        radioMap.setText("Mapped");
        buttonMap.setText("Map Table...");
        checkStatic.setText("Copy to output as a static resource");
        panelClassName.add(textClassName,
                           new GridBagConstraints(0, 1, 4,
                                                  GridBagConstraints.REMAINDER,
                                                  1.0, 1.0,
                                                  GridBagConstraints.WEST,
                                                  GridBagConstraints.HORIZONTAL,
                                                  new Insets(5, 5, 5, 5), 0,
                                                  0));
        panelClassName.add(radioCustom,
                           new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0,
                                                  GridBagConstraints.CENTER,
                                                  GridBagConstraints.NONE,
                                                  new Insets(0, 0, 0, 0), 0,
                                                  0));
        panelClassName.add(radioMap,
                           new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0,
                                                  GridBagConstraints.CENTER,
                                                  GridBagConstraints.NONE,
                                                  new Insets(0, 0, 0, 0), 0,
                                                  0));
        panelClassName.add(radioDefault,
                           new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
                                                  GridBagConstraints.CENTER,
                                                  GridBagConstraints.NONE,
                                                  new Insets(0, 0, 0, 0), 0,
                                                  0));
        panelClassName.add(buttonMap,
                           new GridBagConstraints(3, 0, 1, 1, 0.0, 0.0,
                                                  GridBagConstraints.CENTER,
                                                  GridBagConstraints.NONE,
                                                  new Insets(0, 0, 0, 0), 0,
                                                  0));
        this.setLayout(layoutMain);
        this.add(checkCompile,
                 new GridBagConstraints(0, 1, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.WEST,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(2, 5, 5, 5), 0, 0));
        this.add(checkKeep,
                 new GridBagConstraints(0, 2, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.WEST,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(5, 20, 5, 5), 0, 0));
        this.add(panelClassName,
                 new GridBagConstraints(0, 3, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(3, 15, 3, 5), 0, 0));
        this.add(optionTab,
                 new GridBagConstraints(0, 4, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(3, 15, 5, 5), 0, 0));
        this.add(checkStatic,
                 new GridBagConstraints(0, 0, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.WEST,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(5, 5, 2, 5), 0, 0));
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    public void readProperties() {
        boolean b = false;
        String  s = new String();

        optionTab.setNode(node);
        optionTab.init();
        optionTab.readProperties();
        b = node.isSelected();
        checkCompile.setSelected(b);
        if (b) {
            checkStatic.setSelected(false);
        } else {
            b = node.isStatic();
            checkStatic.setSelected(b);
        }
        if (node.getClassNameType() == OtterXMLCNode.CLASS_NAME_CUSTOM) {
            radioCustom.setSelected(true);
        } else if (node.getClassNameType()
                   == OtterXMLCNode.CLASS_NAME_MAPPED) {
            radioMap.setSelected(true);
        } else {
            radioDefault.setSelected(true);
        }
        s = node.getMetaDataHandler().getClassName();
        textClassName.setText(s);
        checkKeep.setSelected(true);
        checkKeep.setEnabled(false);
        enableCompileOptions(checkCompile.isSelected()
                             && (!checkStatic.isSelected()));
        radioSelection();
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    public void writeProperties() {
        optionTab.writeProperties();
        boolean b = checkCompile.isSelected();

        node.setSelected(b);
        if (b) {
            node.setStatic(false);
        } else {
            b = checkStatic.isSelected();
            node.setStatic(b);
        }
        node.getMetaDataHandler().setKeepGeneratedSource(checkKeep.isSelected());
        String customClassName = textClassName.getText();

        node.setCustomClassName(customClassName);
        if (radioCustom.isSelected()) {
            node.setClassNameType(OtterXMLCNode.CLASS_NAME_CUSTOM);
        } else if (radioDefault.isSelected()) {
            node.setClassNameType(OtterXMLCNode.CLASS_NAME_DEFAULT);
        } else {
            node.setClassNameType(OtterXMLCNode.CLASS_NAME_MAPPED);
        }
    }

    /**
     * Method declaration
     *
     *
     * @param enable
     *
     * @see
     */
    private void enableCompileOptions(boolean enable) {
        radioDefault.setEnabled(enable);
        radioCustom.setEnabled(enable);
        radioMap.setEnabled(enable);
        textClassName.setEnabled(radioCustom.isSelected());
        checkKeep.setEnabled(false);
        textClassName.setBackground(UIManager.getColor(textClassName.isEnabled()
                ? "TextField.background" : "Label.background"));
        buttonMap.setEnabled(enable);
        optionTab.setEnabled(enable);
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    private void radioSelection() {
        textClassName.setEnabled(radioCustom.isSelected());
        textClassName.setBackground(UIManager.getColor(textClassName.isEnabled()
                ? "TextField.background" : "Label.background"));
        if (node != null) {
            writeProperties();
            String s = node.getMetaDataHandler().getClassName();

            textClassName.setText(s);
        }
        buttonMap.setEnabled(radioMap.isSelected());
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    private void editMap() {
        MapDialog dialog =
            new MapDialog(((JDialog) this.getTopLevelAncestor()),
                          "Edit Project Map", true);

        dialog.setProject(node.getProject());
        dialog.show();
    }

    /**
     * Class declaration
     *
     *
     * @author Paul Mahar
     * @version $Revision: 1.6 $
     */
    private class LocalButtonListener implements ActionListener {

        /**
         * Method declaration
         *
         *
         * @param e
         *
         * @see
         */
        public void actionPerformed(ActionEvent e) {
            Object source = e.getSource();

            if (source == buttonMap) {
                editMap();
            }
        }

    }

    private class LocalRadioListener implements ChangeListener {
       public void stateChanged(ChangeEvent e) {
         JRadioButton source = (JRadioButton) e.getSource();
         if (source.isSelected()) {
           radioSelection();
         }
       }
    }

    /**
     * Class declaration
     *
     *
     * @author Paul Mahar
     * @version $Revision: 1.6 $
     */
    private class LocalCheckListener implements ActionListener {

        /**
         * Method declaration
         *
         *
         * @param e
         *
         * @see
         */
        public void actionPerformed(ActionEvent e) {
            if (e.getSource() == checkCompile && checkCompile.isSelected()) {
                checkStatic.setSelected(false);
            } else if (e.getSource() == checkStatic
                       && checkStatic.isSelected()) {
                checkCompile.setSelected(false);
            }
            enableCompileOptions(checkCompile.isSelected()
                                 && (!checkStatic.isSelected()));
        }

    }
}
