/*
 * Enhydra Java Application Server
 * The Initial Developer of the Original Code is Lutris Technologies Inc.
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc.
 * All Rights Reserved.
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License.
 *
 * $Id: AbstractCodeGenWizard.java,v 1.4 2000/08/01 07:09:13 pmahar Exp $
 */
package org.enhydra.kelp.jbuilder.wizard;

// Kelp imports
import org.enhydra.kelp.common.wizard.codegen.CodeGenUtil;
import org.enhydra.kelp.common.wizard.codegen.CodeGenException;
import org.enhydra.kelp.common.swing.SwingUtil;
import org.enhydra.kelp.common.swing.IconSet;
import org.enhydra.kelp.KelpVersion;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.OtterNodeFactory;
import org.enhydra.kelp.jbuilder.node.PrimeProject;
import org.enhydra.kelp.jbuilder.node.PrimeProject;
import org.enhydra.kelp.common.node.OtterFolderNode;

// CodeGen imports
import org.enhydra.codegen.wizard.CodeGenPanel;

// JBuilder Foundation imports
import com.borland.primetime.wizard.BasicWizard;
import com.borland.primetime.wizard.BasicWizardPage;
import com.borland.primetime.wizard.Wizard;
import com.borland.primetime.wizard.WizardAction;
import com.borland.primetime.wizard.WizardDialog;
import com.borland.primetime.wizard.WizardHost;
import com.borland.primetime.wizard.WizardManager;
import com.borland.primetime.wizard.WizardPage;
import com.borland.primetime.util.VetoException;
import com.borland.primetime.ide.BrowserIcons;
import com.borland.primetime.vfs.Url;
import com.borland.primetime.node.Node;
import com.borland.primetime.node.Project;
import com.borland.primetime.node.DuplicateNodeException;
import com.borland.primetime.help.ZipHelpTopic;
import com.borland.jbuilder.wizard.project.JBProjectWizard;
import com.borland.jbuilder.node.JBProject;
import com.borland.jbuilder.node.HTMLFileNode;
import com.borland.jbuilder.build.JavaBuildPropertyGroup;

// Standard imports
import java.awt.BorderLayout;
import java.io.File;
import java.util.Properties;
import javax.swing.JOptionPane;
import javax.swing.ImageIcon;

/**
 * Class declaration
 *
 *
 * @author
 * @version %I%, %G%p
 */
abstract public class AbstractCodeGenWizard extends BasicWizard {
    private CodeGenUtil util = null;
    private JBProject   nativeProject = null;

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public abstract String getHelpTopic();

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public abstract String getDisplayName();

    /**
     * Method declaration
     *
     *
     * @param currentPage
     * @param host
     *
     * @see
     */
    public void help(WizardPage currentPage, WizardHost host) {
        ZipHelpTopic t = new ZipHelpTopic(null, getHelpTopic());

        t.show(host.getDialogParent());
    }

    /**
     * Method declaration
     *
     *
     * @param host
     *
     * @return
     *
     * @see
     */
    abstract public WizardPage invokeWizard(WizardHost host);

    /**
     * Method declaration
     *
     * @param host
     *
     * @return
     *
     * @see
     */
    public WizardPage invokeWizard(WizardHost host, CodeGenUtil util) {
        WizardPage   wizardPage = null;
        boolean      invoke = true;

        this.util = util;
        if (!isProjectEmpty()) {
            int choice =
                JOptionPane.showConfirmDialog(host.getDialogParent(),
                 "Project already contains entries. Importing \n" +
                 "files into an non-empty project may cause \n" +
                 "unexpected results. \n\n" +
                 "Do you want to continue?",
                                              getDisplayName()
                                              + " Wizard Error", JOptionPane.WARNING_MESSAGE);

            if (choice == JOptionPane.CANCEL_OPTION) {
                invoke = false;
            }
        }
        if (invoke) {
            try {
                setWizardTitle(getDisplayName() + " Wizard");
                if ((nativeProject != null)
                        && (nativeProject instanceof JBProject)) {
                    for (int i = 0; i < util.getWizardPanels().length; i++) {
                        LocalWizardPage page = new LocalWizardPage();

                        page.setPanel(util.getWizardPanels()[i]);
                        addWizardPage(page);
                    }
                    wizardPage = super.invokeWizard(host);
                }
            } catch (CodeGenException e) {
                JOptionPane.showMessageDialog(host.getDialogParent(),
                                              e.getMessage(),
                                              getDisplayName()
                                              + " Wizard Error", JOptionPane.WARNING_MESSAGE);
            }
        }
        return wizardPage;
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    private boolean isProjectEmpty() {
        Node[]  nodes = null;
        boolean empty = true;

        if (nativeProject != null) {
            nodes = nativeProject.getChildren();
        }
        if (nodes != null) {
            if (nodes.length > 1) {
                empty = false;
            } else if (nodes.length == 1) {
                if (!(nodes[0] instanceof HTMLFileNode)) {
                    empty = false;
                }
            }
        }
        return empty;
    }

    /**
     * Method declaration
     *
     *
     * @param host
     *
     * @see
     */
    protected OtterProject initProject(WizardHost host) {
        OtterProject otterProj = null;
        if (host.getBrowser().getActiveProject() == null) {
          nativeProject = null;
        } else if (host.getBrowser().getActiveProject() instanceof JBProject) {
          nativeProject = (JBProject) host.getBrowser().getActiveProject();
        }
        if (nativeProject == null) {
            WizardDialog dlg = null;
            JBProjectWizard wiz = null;
            wiz = new JBProjectWizard();
            dlg = new WizardDialog(host.getBrowser(), wiz);
            dlg.show();
            if (host.getBrowser().getActiveProject() == null) {
                nativeProject = null;
            } else if (host.getBrowser().getActiveProject() instanceof JBProject) {
               nativeProject = (JBProject) host.getBrowser().getActiveProject();
            }
        }
        if (nativeProject != null) {
           if (nativeProject instanceof JBProject) {
             otterProj = new PrimeProject((JBProject)nativeProject);
            }
        }
        return otterProj;
    }

    /**
     * Method declaration
     *
     *
     * @throws VetoException
     *
     * @see
     */
    public WizardPage finish(WizardPage currentPage,
                             WizardHost host) throws VetoException {
        File[]  files = new File[0];
        boolean continueBuild = true;

        if (nativeProject != null && util != null) {
            try {
                files = util.generate();
                util.setupProject(files);
                nativeSetupProject();
            } catch (CodeGenException e) {
                JOptionPane.showMessageDialog(this.wizardHost.getDialogParent(),
                                              e.getMessage(),
                                              getDisplayName()
                                              + " Wizard Error", JOptionPane.WARNING_MESSAGE);
                throw new VetoException();
            }
        }
        return null;
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    private void nativeSetupProject() {
        File         sourceDir = null;
        OtterProject otterProject = null;

        otterProject = new PrimeProject(nativeProject);
        otterProject.configureRunClass();
        if (!nativeProject.getPaths().putClassOnFullPath(KelpVersion.getMultiServerRunClass())) {

            // no enhydra library
        }
        JavaBuildPropertyGroup.SYNC_DIR.setValue(nativeProject,
                                                 String.valueOf(false));
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    abstract public String[] getSteps();

    /**
     * Class declaration
     *
     *
     * @author Paul Mahar
     * @version $Revision: 1.4 $
     */
    class LocalWizardPage extends BasicWizardPage {
        private CodeGenPanel panel;

        /**
         * Constructor declaration
         *
         *
         * @see
         */
        public LocalWizardPage() {
            BorderLayout layout = new BorderLayout();

            layout.setHgap(5);
            layout.setVgap(5);
            this.setLayout(layout);
            this.setLargeIcon(IconSet.getOtterWizard());
        }

        /**
         * Method declaration
         *
         *
         * @param host
         *
         * @see
         */
        public void activated(WizardHost host) {
            host.setFinishEnabled(true);
        }

        /**
         * Constructor declaration
         *
         *
         * @param panel
         *
         * @see
         */
        protected void setPanel(CodeGenPanel panel) {
            this.panel = panel;
            this.setPageTitle(panel.getPageTitle());
            this.setInstructions(panel.getInstructions());
            this.add(panel, BorderLayout.CENTER);
        }

        /**
         * Constructor declaration
         *
         *
         * @see
         */
        protected CodeGenPanel getPanel() {
            return panel;
        }

    }
}
