/*
 * Enhydra Java Application Server
 * The Initial Developer of the Original Code is Lutris Technologies Inc.
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc.
 * All Rights Reserved.
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License.
 *
 * $Id: OracleProject.java,v 1.10 2000/07/28 23:49:03 pmahar Exp $
 */
package org.enhydra.kelp.jdeveloper.node;

// Kelp imports
import org.enhydra.kelp.KelpVersion;
import org.enhydra.kelp.common.wizard.conf.ConfBuildThread;
import org.enhydra.kelp.common.node.OtterImageFileNode;
import org.enhydra.kelp.common.node.OtterTextFileNode;
import org.enhydra.kelp.common.node.OtterHTMLFileNode;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.node.OtterNodeFactory;
import org.enhydra.kelp.common.node.OtterWMLFileNode;

// JDeveloper imports
import borland.jbuilder.addin.Container;
import borland.jbuilder.addin.HtmlNode;
import borland.jbuilder.addin.Node;
import borland.jbuilder.addin.Project;
import borland.jbuilder.addin.Url;

// Standard imports
import java.util.Vector;
import java.io.File;

/**
 * Class declaration
 *
 *
 * @author
 * @version %I%, %G%
 */
public class OracleProject extends OtterProject implements OtterNode {
    private Project           nativeProject = null;
    private OracleNodeFactory factory = null;

    public String getCodeGenDefaultDestination() {
       String dest = new String();
       File file = null;
       file = new File(getFilePath());
       dest = file.getParent();
       return dest;
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public OtterTextFileNode getFirstPolicy() {
        PolicyNodeReader reader = new PolicyNodeReader();

        return reader.getFirstNode(nativeProject);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public OtterImageFileNode[] getAllImages() {
        ImageNodeReader reader = new ImageNodeReader();

        return reader.getNodes(nativeProject);
    }

    /**
     * Method declaration
     *
     *
     * @param extensions
     *
     * @return
     *
     * @see
     */
    public OtterTextFileNode[] getAllText(String[] extensions) {
        TextNodeReader reader = new TextNodeReader(extensions);

        return reader.getNodes(nativeProject);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public OtterTextFileNode[] getAllDeploymentDescs() {
        DeploymentNodeReader reader = new DeploymentNodeReader();

        return reader.getNodes(nativeProject);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public OtterHTMLFileNode[] getAllHTMLFiles() {
        HTMLNodeReader reader = new HTMLNodeReader();

        return reader.getNodes(nativeProject);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public OtterWMLFileNode[] getAllWMLFiles() {
        WMLNodeReader reader = new WMLNodeReader();

        return reader.getNodes(nativeProject);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String getName() {
        String name = null;

        name = nativeProject.getName();
        if (name.startsWith(nativeProject.getPath())) {
            name = name.substring(nativeProject.getPath().length() + 1);
        }
        if (name.indexOf(".") > 0) {
            name = name.substring(0, name.indexOf("."));
        }
        return name;
    }

    /**
     * Constructor declaration
     *
     *
     * @param p
     *
     * @see
     */
    public OracleProject() {
       factory = new OracleNodeFactory();
    }

    public OracleProject(Project p) {
        this();
        setNativeProject(p);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public OtterNodeFactory getNodeFactory() {
        return factory;
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public Object getNativeProject() {
        return nativeProject;
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public OtterProject getProject() {
        return this;
    }

    public OtterNode getParent() {
      return null;
    }

    /**
     * Method declaration
     *
     *
     * @param p
     *
     * @see
     */
    public void setNativeProject(Object p) {
        nativeProject = (Project) p;
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public Object getNativeNode() {
        return getNativeProject();
    }

    /**
     * Method declaration
     *
     *
     * @param n
     *
     * @see
     */
    public void setNativeNode(Object n) {
        setNativeProject(n);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public boolean isOpenBuild() {
        return false;
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String getClassOutputPath() {
        String out = null;

        if (nativeProject != null) {
            out = nativeProject.getOutputDirectory();
        }
        return out;
    }

    public void setClassOutputPath(String p) {
      nativeProject.BeginUpdate();
      nativeProject.setOutputDirectory(p);
      nativeProject.save(true);
      nativeProject.EndUpdate();
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String getSourcePath() {
        String out = null;

        if (nativeProject != null) {
            out = nativeProject.getProjectSourcePath();
        }
        return out;
    }

    /**
     * Method declaration
     *
     *
     * @param pathAsArray
     *
     * @see
     */
    public void setSourcePathArray(String[] pathAsArray) {
        StringBuffer path = new StringBuffer();

        for (int i = 0; i < pathAsArray.length; i++) {
            path.append(pathAsArray[i] + ";");
        }
        nativeProject.BeginUpdate();
        nativeProject.setProjectSourcePath(path.toString());
        nativeProject.save(true);
        nativeProject.EndUpdate();
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String[] getSourcePathArray() {
        String[] pathAsArray = null;
        String   pathAsString = getSourcePath();
        Vector   pathAsVector = new Vector();
        int      at = pathAsString.indexOf(";");
        int      size = 0;

        while (at > -1) {
            pathAsVector.addElement(pathAsString.substring(0, at).trim());
            pathAsString = pathAsString.substring(at + 1).trim();
            at = pathAsString.indexOf(";");
        }
        pathAsVector.addElement(pathAsString.trim());
        size = pathAsVector.size();
        if (size > 0) {
            pathAsArray = new String[size];
            for (int i = 0; i < size; i++) {
                pathAsArray[i] = (String) pathAsVector.elementAt(i);
            }
        }
        return pathAsArray;
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String getClassPath() {
        String out = null;

        if (nativeProject != null) {
            out = nativeProject.getClassPath();
        }
        return out;
    }

    /**
     * Method declaration
     *
     *
     * @param property
     *
     * @return
     *
     * @see
     */
    public String getProperty(String property) {
        String out = null;

        if (nativeProject != null) {
            out = nativeProject.getProperty(property);
        }
        return out;
    }

    /**
     * Method declaration
     *
     *
     * @param property
     * @param value
     *
     * @see
     */
    public void setProperty(String property, String value) {
        if (nativeProject != null) {
            nativeProject.BeginUpdate();
            if (property.trim().equalsIgnoreCase("Libraries")) {
              String[] libs = new String[1];
              libs[0] = value;
              nativeProject.setLibraries(libs);
            } else {
              nativeProject.setProperty(property, value);
            }
            nativeProject.save(true);
            nativeProject.EndUpdate();
        }
    }

    /**
     * Method declaration
     *
     *
     * @param property
     * @param i
     *
     * @see
     */
    public void setProperty(String property, int i) {
        String value = Integer.toString(i);

        setProperty(property, value);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String getFilePath() {
        return nativeProject.getUrl().getFilename();
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public char getFileSeparatorChar() {
        return File.separatorChar;
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    public void configureRunClass() {
        StringBuffer policyPath = new StringBuffer();
        policyPath.append(getConfStartupConf());
        policyPath.append(File.separator);
        policyPath.append("java.policy");

        File policy = new File(policyPath.toString());

        if (!policy.exists()) {
           policy = ConfBuildThread.createPolicyFile(policyPath.toString());
        }

        StringBuffer value = new StringBuffer();
        value.append(getSourcePathArray()[0]);
        value.append(File.separator);
        value.append("StartMultiServer.java");
        nativeProject.setProperty("DefaultRunnablePath", value.toString());

        if (KelpVersion.getEnhydraVersion().startsWith("3")) {
            value.setLength(0);
            value.append("\"");
            value.append(getConfStartupConf());
            value.append("\"");
            nativeProject.setProperty("RunParameters", value.toString());
        } else {
            value.setLength(0);
            value.append(" -Djava.security.policy=");
            value.append("\"");
            value.append(policyPath.toString());
            value.append("\"");
            value.append(" -Denhydra.conf=");
            value.append("\"");
            value.append(getConfStartupConf());
            value.append("\"");
            nativeProject.setProperty("JavaVmParameters", value.toString());

            value.setLength(0);
            value.append("\"");
            value.append(getConfStartupConf());
            value.append(File.separator);
            value.append("servlet");
            value.append(File.separator);
            value.append("kernel.conf");
            value.append("\"");
            nativeProject.setProperty("RunParameters", value.toString());
        }
    }

}

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 * @version $Revision: 1.10 $
 */
class ImageNodeReader {
    private Vector nodeVector = new Vector();

    /**
     * Constructor declaration
     *
     *
     * @see
     */
    protected ImageNodeReader() {}

    /**
     * Method declaration
     *
     *
     * @param nativeProject
     *
     * @return
     *
     * @see
     */
    protected OtterImageFileNode[] getNodes(Project nativeProject) {
        OtterImageFileNode[] nodes;

        recursiveRead(nativeProject);
        nodes = new OtterImageFileNode[nodeVector.size()];
        for (int i = 0; i < nodes.length; i++) {
            nodes[i] = (OtterImageFileNode) nodeVector.elementAt(i);
        }
        return nodes;
    }

    /**
     * Method declaration
     *
     *
     * @param container
     *
     * @see
     */
    private void recursiveRead(Container container) {
        String filename = null;
        Node[] nativeNodes = container.getChildren();
        File   current = null;

        for (int i = 0; i < nativeNodes.length; i++) {
            if (nativeNodes[i] instanceof Container) {
                if (!nativeNodes[i].toString().equalsIgnoreCase("RootContainer")) {
                    recursiveRead((Container) nativeNodes[i]);
                }
            } else {
                filename = nativeNodes[i].getFilename();
                current = new File(filename);
                if ((filename.toLowerCase().endsWith(".gif") || filename.toLowerCase().endsWith(".jpg"))
                        && current.exists()) {
                    OtterImageFileNode imageNode = null;

                    imageNode = new OracleImageFileNode(nativeNodes[i]);
                    nodeVector.addElement(imageNode);
                }
            }
        }
    }

}

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 * @version $Revision: 1.10 $
 */
class DeploymentNodeReader {
    private Vector nodeVector = new Vector();

    /**
     * Constructor declaration
     *
     *
     * @see
     */
    protected DeploymentNodeReader() {}

    /**
     * Method declaration
     *
     *
     * @param nativeProject
     *
     * @return
     *
     * @see
     */
    protected OtterTextFileNode[] getNodes(Project nativeProject) {
        OtterTextFileNode[] nodes;

        recursiveRead(nativeProject);
        nodes = new OtterTextFileNode[nodeVector.size()];
        for (int i = 0; i < nodes.length; i++) {
            nodes[i] = (OtterTextFileNode) nodeVector.elementAt(i);
        }
        return nodes;
    }

    /**
     * Method declaration
     *
     *
     * @param container
     *
     * @see
     */
    private void recursiveRead(Container container) {
        String filename = null;
        Node[] nativeNodes = container.getChildren();
        File   current = null;

        for (int i = 0; i < nativeNodes.length; i++) {
            if (nativeNodes[i] instanceof Container) {
                if (!nativeNodes[i].toString().equalsIgnoreCase("RootContainer")) {
                    recursiveRead((Container) nativeNodes[i]);
                }
            } else {
                filename = nativeNodes[i].getFilename();
                current = new File(filename);
                if ((filename.toLowerCase().endsWith("web.xml") || filename.toLowerCase().endsWith("ejb-jar.xml"))
                        && current.exists()) {
                    OtterTextFileNode imageNode = null;

                    imageNode = new OracleTextFileNode(nativeNodes[i]);
                    nodeVector.addElement(imageNode);
                }
            }
        }
    }

}

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 * @version $Revision: 1.10 $
 */
class HTMLNodeReader {
    private Vector nodeVector = new Vector();

    /**
     * Constructor declaration
     *
     *
     * @see
     */
    protected HTMLNodeReader() {}

    /**
     * Method declaration
     *
     *
     * @param nativeProject
     *
     * @return
     *
     * @see
     */
    protected OtterHTMLFileNode[] getNodes(Project nativeProject) {
        OtterHTMLFileNode[] nodes;

        recursiveRead(nativeProject);
        nodes = new OtterHTMLFileNode[nodeVector.size()];
        for (int i = 0; i < nodes.length; i++) {
            nodes[i] = (OtterHTMLFileNode) nodeVector.elementAt(i);
        }
        return nodes;
    }

    /**
     * Method declaration
     *
     *
     * @param container
     *
     * @see
     */
    private void recursiveRead(Container container) {
        String filename = null;
        Node[] nativeNodes = container.getChildren();
        File   current = null;

        for (int i = 0; i < nativeNodes.length; i++) {
            if (nativeNodes[i] instanceof Container) {
                if (!nativeNodes[i].toString().equalsIgnoreCase("RootContainer")) {
                    recursiveRead((Container) nativeNodes[i]);
                }
            } else if (nativeNodes[i] instanceof HtmlNode) {
                filename = nativeNodes[i].getFilename();
                current = new File(filename);
                if ((filename.toLowerCase().endsWith(".html"))
                        && current.exists()) {
                    OtterHTMLFileNode newNode = null;

                    newNode =
                        new OracleHTMLFileNode((HtmlNode) nativeNodes[i]);
                    nodeVector.addElement(newNode);
                }
            }
        }
    }

}

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 * @version $Revision: 1.10 $
 */
class WMLNodeReader {
    private Vector nodeVector = new Vector();

    /**
     * Method declaration
     *
     *
     * @param nativeProject
     *
     * @return
     *
     * @see
     */
    protected OtterWMLFileNode[] getNodes(Project nativeProject) {
        OtterWMLFileNode[] nodes;

        recursiveRead(nativeProject);
        nodes = new OtterWMLFileNode[nodeVector.size()];
        for (int i = 0; i < nodes.length; i++) {
            nodes[i] = (OtterWMLFileNode) nodeVector.elementAt(i);
        }
        return nodes;
    }

    /**
     * Method declaration
     *
     *
     * @param container
     *
     * @see
     */
    private void recursiveRead(Container container) {
        String filename = null;
        Node[] nativeNodes = container.getChildren();
        File   current = null;

        for (int i = 0; i < nativeNodes.length; i++) {
            if (nativeNodes[i] instanceof Container) {
                if (!nativeNodes[i].toString().equalsIgnoreCase("RootContainer")) {
                    recursiveRead((Container) nativeNodes[i]);
                }
            } else {
                filename = nativeNodes[i].getFilename();
                current = new File(filename);
                if ((filename.toLowerCase().endsWith(".wml"))
                        && current.exists()) {
                    OtterWMLFileNode newNode = null;

                    newNode = new OracleWMLFileNode(nativeNodes[i]);
                    nodeVector.addElement(newNode);
                }
            }
        }
    }

}

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 * @version $Revision: 1.10 $
 */
class TextNodeReader {
    private Vector   nodeVector = new Vector();
    private String[] extensions = new String[0];

    /**
     * Constructor declaration
     *
     *
     * @param exts
     *
     * @see
     */
    protected TextNodeReader(String[] exts) {
        extensions = exts;
    }

    /**
     * Method declaration
     *
     *
     * @param nativeProject
     *
     * @return
     *
     * @see
     */
    protected OtterTextFileNode[] getNodes(Project nativeProject) {
        OtterTextFileNode[] nodes;

        recursiveRead(nativeProject);
        nodes = new OtterTextFileNode[nodeVector.size()];
        for (int i = 0; i < nodes.length; i++) {
            nodes[i] = (OtterTextFileNode) nodeVector.elementAt(i);
        }
        return nodes;
    }

    /**
     * Method declaration
     *
     *
     * @param container
     *
     * @see
     */
    private void recursiveRead(Container container) {
        String filename = null;
        Node[] nativeNodes = container.getChildren();
        File   current = null;

        for (int i = 0; i < nativeNodes.length; i++) {
            if (nativeNodes[i] instanceof Container) {
                if (!nativeNodes[i].toString().equalsIgnoreCase("RootContainer")) {
                    recursiveRead((Container) nativeNodes[i]);
                }
            } else {
                filename = nativeNodes[i].getFilename();
                current = new File(filename);
                for (int j = 0; j < extensions.length; j++) {
                    if ((filename.toLowerCase().endsWith("." + extensions[j]))
                            && current.exists()) {
                        OtterTextFileNode newNode = null;

                        newNode = new OracleTextFileNode(nativeNodes[i]);
                        nodeVector.addElement(newNode);
                        break;
                    }
                }
            }
        }
    }

}

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 * @version $Revision: 1.10 $
 */
class PolicyNodeReader {
    private OtterTextFileNode node = null;

    /**
     * Method declaration
     *
     *
     * @param nativeProject
     *
     * @return
     *
     * @see
     */
    protected OtterTextFileNode getFirstNode(Project nativeProject) {
        recursiveRead(nativeProject);
        return node;
    }

    /**
     * Method declaration
     *
     *
     * @param container
     *
     * @see
     */
    private void recursiveRead(Container container) {
        String filename = null;
        Node[] nativeNodes = container.getChildren();
        File   current = null;

        for (int i = 0; i < nativeNodes.length; i++) {
            if (nativeNodes[i] instanceof Container) {
                if (!nativeNodes[i].toString().equalsIgnoreCase("RootContainer")) {
                    recursiveRead((Container) nativeNodes[i]);
                }
            } else {
                filename = nativeNodes[i].getFilename();
                current = new File(filename);
                if ((filename.toLowerCase().endsWith("java.policy"))
                        && current.exists()) {
                    node = new OracleTextFileNode(nativeNodes[i]);
                    break;
                }
            }
        }
    }

}
