/*
 * Enhydra Java Application Server
 * The Initial Developer of the Original Code is Lutris Technologies Inc.
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc.
 * All Rights Reserved.
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License.
 *
 * $Id: XMLCWizard.java,v 1.4 2000/07/28 01:03:14 pmahar Exp $
 */

package org.enhydra.kelp.jdeveloper.wizard.xmlc;

import org.enhydra.kelp.common.swing.IconSet;
import org.enhydra.kelp.common.swing.SwingUtil;
import org.enhydra.kelp.KelpVersion;
import org.enhydra.kelp.common.node.OtterXMLCNode;
import org.enhydra.kelp.common.wizard.xmlc.XMLCBuildDialog;
import org.enhydra.kelp.jdeveloper.node.OracleProject;
import org.enhydra.kelp.jdeveloper.node.OracleHTMLFileNode;
import borland.jbuilder.addin.Container;
import borland.jbuilder.addin.Context;
import borland.jbuilder.addin.IdeEnvironment;
import borland.jbuilder.addin.VersionedAddin;
import borland.jbuilder.addin.Wizard;
import borland.jbuilder.addin.WizardManager;
import borland.jbuilder.addin.Node;
import borland.jbuilder.addin.HtmlNode;
import borland.jbuilder.addin.Project;

// Standard imports
import javax.swing.JOptionPane;
import java.awt.Component;
import java.awt.Dialog;
import java.awt.Frame;
import java.awt.Image;
import java.io.File;
import java.io.Serializable;
import java.util.Vector;
import java.lang.System;
///
import java.net.URLClassLoader;
import java.net.URL;
import java.io.*;
import java.util.*;
import java.lang.reflect.*;
import java.security.*;
/**
 * This is the class that gets registered with the IDE through the
 * jbuilder.properties file.
 */
public class XMLCWizard implements Serializable, VersionedAddin, Wizard {
    private String       title = null;
    private int		 option = 0;

    /**
     * Constructor declaration
     *
     *
     * @see
     */
    public XMLCWizard() {}

    /**
     * For the VersionAddin Interface.
     */
    public String version() {
	return "2.0";
    }

    /**
     * Entry point exposed by the Wizard interface for early initialization.
     * Registers this Wizard object with the IDE's WizardManager object
     * @$param ide Access to all AddIn interfaces
     */
    public void startup(IdeEnvironment ide) {
	WizardManager e = ide.getWizardManager();
	e.registerWizard(this);
    }

    /**
     * Wizard interface implementation, run the wizard in response to this call.
     * @param ide Access to all AddIn interfaces
     * @param context Current node, project, browser
     */
    public void invoke(IdeEnvironment ide, Context context) {
        Project      nativeProject = null;
	Frame	     frame = null;
	frame = ide.getIdeFrame();
	nativeProject = context.getProject();

        String ver = System.getProperty("java.version");
        if (ver.startsWith("1.1")) {
          JOptionPane.showMessageDialog(frame,
                                        "Kelp requires Java 2.\n" +
                                        "Found version: " + ver);
          nativeProject = null;
        }

	if (nativeProject != null
		&& isEnhydraAvailable(ide, nativeProject.getClassPath())) {
            nativeProject.save(false);    // save all file changes in project

            try {
              XMLCBuildDialog dlg = null;

              // Create Dialog
              dlg = new XMLCBuildDialog(frame);
              dlg.setModal(true);
              dlg.setTitle(getName());

              // Create Project Node
              OracleProject oracleProject = null;
              oracleProject = new OracleProject();
              oracleProject.setNativeNode(nativeProject);

              // show dialog
	      dlg.setProject(oracleProject);
	      dlg.show();
            } catch (Exception e) {
              e.printStackTrace();
              JOptionPane.showMessageDialog(ide.getIdeFrame(),
                                        "Unable to load wizard.\n" +
                                        e.getMessage());
            }
	    frame.requestFocus();
	}
    }

    /**
     * Wizard interface method, returns information about the type of Wizard this is.
     * @return An integer representing combined flags defined by the
     * Wizard interface.
     */
    public int getInfo() {
	return Wizard.PRODUCES_FILE + Wizard.REQUIRES_PROJECT;
    }

    /**
     * Wizard interface method, returns the name of this wizard.
     * @return Name of the wizard
     */
    public String getName() {
	return "XMLC Compiler";
    }

    /**
     * Wizard interface method, returns the text for the menu label
     * @return Menu label text
     */
    public java.lang.String getMenuLabel() {
	return getName() + "...";
    }

    /**
     * Wizard interface method
     * @return Currently returns no image (null)
     */
    public Image getSmallImage() {
	return IconSet.getSmallIcon().getImage();
    }

    /**
     * Wizard interface method
     * @return Currently returns no image (null)
     */
    public Image getBigImage() {
	return IconSet.getLargeIcon().getImage();
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String getTitle() {
	return this.title;
    }

    /**
     * Method declaration
     *
     *
     * @param newTitle
     *
     * @see
     */
    public void setTitle(String newTitle) {
	this.title = newTitle;
    }

    // read only properties

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public int getOption() {
	return option;
    }

    /**
     * Method declaration
     *
     *
     * @param newOption
     *
     * @see
     */
    private void setOption(int newOption) {
	option = newOption;
    }


    /**
     * Method declaration
     *
     *
     * @param ide
     * @param projectClassPath
     *
     * @return
     *
     * @see
     */
    private boolean isEnhydraAvailable(IdeEnvironment ide,
				       String projectClassPath) {
	String       message = null;

	if (! KelpVersion.isEnhydraInClassPath()) {
	    message =
		"Enhydra not found in IDE class path. \n"
		+ "Please verify that the enhydra.jar file is "
                + "included the IDEClassPath setting of "
		+ "the JDEVELOPER.INI file.\n\n";
	}

	if (message == null) {
	    if (projectClassPath.toLowerCase().indexOf("enhydra.jar") < 0) {
		message =
		    "The enhydra.jar file is not included in the "
		    + "current project library set.\n"
		    + "Select the Project | Properties menu "
		    + "to add it before using XMLC.\n\n";
	    }
	}

	if (message != null) {
	    JOptionPane.showMessageDialog(null, message);
	}

	return (message == null);
    }


}
